//
// Created by servostar on 09.12.23.
//

#include "uart.h"

void uart_init(uint16_t baud) {
    uint16_t ubrr = F_CPU / 16 / baud - 1;

    UBRR0H = (uint8_t) (ubrr >> 8);
    UBRR0L = (uint8_t) (ubrr & 0xFF);

    UCSR0B |= (1 << RXEN0) | (1 << TXEN0);  // enable receiver and transmitter
    UCSR0C |= (1 << UCSZ01) | (3 << UCSZ00); // set frame: 8 bit
    UCSR0C &= ~(1 << USBS0); // 1 stop bit

    UCSR0B |= (1 << RXCIE0); // enable receiver interrupt
}

void uart_write(uint8_t frame) {
    // wait until TX is free
    while (!(UCSR0A & (1 << UDRE0)));
    // write data
    UDR0 = frame;
}

uint8_t uart_read() {
    // wait until RX is free
    while (!(UCSR0A & (1 << RXC0)));
    // read data
    return UDR0;
}

void uart_read_bytes(uint8_t* buffer, size_t size) {
    for (size_t i = 0; i < size; i++) {
        buffer[i] = uart_read();
    }
}

void uart_write_bytes(uint8_t* buffer, size_t bytes) {
    for (size_t i = 0; i < bytes; i++) {
        uart_write(buffer[i]);
    }
}

void uart_write_string(const char* string) {
    while(*string)
        uart_write(*string++);
}